from ctypes import *

pcap = CDLL("libpcap.so")

# pcap methods arguments definitions
pcap.pcap_lookupdev.argtypes    = [c_char_p]
pcap.pcap_open_live.argtypes    = [c_char_p, c_int, c_int, c_int, c_char_p]
pcap.pcap_open_offline.argtypes = [c_char_p, c_char_p]
pcap.pcap_lookupnet.argtypes    = [c_char_p, c_void_p, c_void_p, c_char_p]
pcap.pcap_compile.argtypes      = [c_void_p, c_void_p]
pcap.pcap_setfilter.argtypes    = [c_void_p, c_void_p]
pcap.pcap_next.argtypes         = [c_void_p, c_void_p]
pcap.pcap_close.argtypes        = [c_void_p]

# Initialise a C string buffer to hold error messages
err = create_string_buffer(256)

class bpf_insn(Structure):
    _fields_ = [("code",    c_uint8),
                ("jt",      c_uint8),
                ("jf",      c_uint8),
                ("k",       c_uint32)]

class bpf_program(Structure):
    _fields_ = [("bf_len",     c_uint32),
                ("bf_insns",   bpf_insn)]
                
class timeval(Structure):
    _fields_ = [("tv_sec",   c_long),
                ("tv_usec",  c_long)]

class pcap_pkthdr(Structure):
    _fields_ = [("ts",      timeval),
                ("caplen",  c_uint32),
                ("len",     c_uint32)]

def lookup_dev():
    global err
    dev = c_char_p(pcap.pcap_lookupdev(err))
    if dev == None:
        return None
    
    return dev.value
    
def lookup_net(dev):
    global err
    net  = c_uint32()
    mask = c_uint32()

    if pcap.pcap_lookupnet(dev, byref(net), byref(mask), err) == -1:
        return None
    
    return net, mask
    
def setfilter(handle, pcap_filter):
    return pcap.pcap_setfilter(handle, byref(pcap_filter))
    
def pcap_compile(handle, filter_str, optimize, net):
    pcap_filter = bpf_program()
    pcap.pcap_compile(handle, byref(pcap_filter), c_char_p(filter_str), optimize, net[0])
    
    return pcap_filter
    
def open_live(device, snaplen, promisc, to_ms):
    return pcap.pcap_open_live(c_char_p(device), snaplen, promisc, to_ms, err)
        
def open_offline(filename):
    global err
    return pcap.pcap_open_offline(filename, err)
    
def get_error():
    global err
    return err.value
    
def getbuffer(handle):
    header = pcap_pkthdr()
    packet = None
    
    while True:
        packet = pcap.pcap_next(handle, byref(header))
        if packet:
            break
    
    return packet, header
    
def close(handle):
    return pcap.pcap_close(handle)

def next(handle, header):
    return pcap.pcap_next(handle, byref(header))
